% This script file contains the MATLAB code, giving an example 
% for an experimental data set on how to use the function. 
% Written by Ruifen Huo 26/02/2004.

% 1. load an experimental data set
% The sample was prepared by Chris Geurts, Oce Technologies 
% (Venlo, Netherlands) and the data were measured by Roland van de
% Molengraaf and Jeroen Pikkemaat at Philips Research, (Eindhoven,
% Netherlands).

load philip2.mat 
% data10 is the original data, and correctx10_2 is the data after 
% correction (just for reference);

% 2. Preprocessing of DOSY NMR data;
% a. baseline correction
n=50;N=81;
[basecorrect_x]=spec_baseline(data10,n,N,20);
plot(basecorrect_x');

% b. align the peak and phase shift;
area1=6164; area2=6361;
barea=[40:80];
[correctx]=fiddle(basecorrect_x,area1,area2,barea);

figure; plot(correctx');
title('a plot of the corrected data set');

% If there is one peak that still needs to be corrected separately, 
% the following function is used;(optional)
range1=[4660 4781]; % define a single peak (e.g. the first peak)
                    % that needs to be aligned; 
[newx]=correc_peak_phase(correctx,range1);  %only correct phase and
                                            %peak shifts;

% c. correct lineshape variation (only used when lineshape
% variations are found). There are two options in the following;
% you can use either of them depending on the quality of the data;
par1=-0.6; par2=0.6;% this parameter can be changed for different data;

%(1) correct the global lineshape variation;
[correctx]=fiddle_lineshape(basecorrect_x,area1,area2,barea,par1,par2);

%(2) correct lineshape variation peak by peak;(recommended) 
% e.g. the two peaks in the following range (range1);
range1=[4660 4781;10340 10561];
[newx]=correc_lineshape(correctx,range1,par1,par2); 
% this corrects lineshape variation as well as peak and phase shifts;
correctx=newx;

% 3. resolve the DOSY data;
% a. find the pure variables by either using the function opa or your
% knowledge of the chemicals ("correctx" can be replaced by "newx");
ncom=6; 
range1=[4700 4750;6260 6280;6430 6455;6570 6615;6805 6845;6895 6930;7440 ...
      7710;8640 8680;9300 9360;9470 9685;10100 10200;10350 10550];
t=max(correctx(1,:))*0.10;
[c0,normc0]=opa(correctx,ncom,range1,t);

% input 6 components but there are only 4 different decay
% curves. By looking at the normalised pure variables plot one can
% pick four different decays as the initial guess of MCR;

% normalize the columns of c0 into unit length
clear normc0
c0=correctx(:,[6268 10135 10418 9597]);
for i=1:size(c0,2)
  normc0(:,i)=c0(:,i)./norm(c0(:,i));
end

% Extra note for finding pure variables:
%****************************************************************************
% if there are too many overlapping peaks in the spectra, then the spectrum *
% containing fewer overlapping peak can be obtained by Savitsky-Golay       *
% smoothing and differentiation.                                            *
% The syntax of the function from the PLS toolbox is                        *
%   [y_hat]=savgol(correctx,5,3,2);                                         *
% then change the sign of y_hat by y_hat=-yhat; then eliminate any negative *
% values and run opa on the new y_hat data;                                 *
% See Windig et al., Anal.Chem. 74 (2002) 1371.                             *
%****************************************************************************

% b. Using MCR-NLR to resolve the data;

order=[1 1 1 1];
cutoff = 0.01;
maxit = 100;
close all    
K=difflist.^2;
[covresid,covresid1,resi,c,s]=mymcr_nlr(correctx,maxit,normc0,K,order,cutoff); 

% c.(optional) if it takes too much time, then it is better to use a smaller
% matrix size; e.g. to take every second variable, use
ncol = size(correctx, 2);
correctx_new = correctx(:, 2:2:ncol);

% run MCR_NLR;
clear normc0
c0=correctx_new(:,[3134 4914 3785 4798]);
for i=1:size(c0,2)
  normc0(:,i)=c0(:,i)./norm(c0(:,i));
end

[c,s,covresid,covresid1,resi]=mymcr_nlr(correctx_new,maxit,normc0,K,order,cutoff);

% run classical MCR to compare the results;
[cmcr,smcr]=dosy_mcr(correctx_new,c0,50);

% End of demo2.
