function [OptPar,sigma,cormat,status,response,DetHessian] = nlr(X,Y,MODELFUNC,XPAR,OPTIONS,DERIVFUNC,P1,P2,P3,P4,P5,P6,P7,P8,P9)
%   function [OptPar,sigma,cormat,status,response,DetHessian] = nlr(X,Y,MODELFUNC,XPAR,OPTIONS,DERIVFUNC,...)
%
%   Purpose:
%   Non-linear least squares to optimize parameters of a model function that is
%   non-linear in its parameters. 
%
%   Description:    
%
%   NLR solves problems of the form:
%
%          length(Y)
%   min  sum {Y(i) - MODELFUNC(X(i), XPAR(i))}.^2   
%            i=1
%
%   where Y is a colomn vector, and X may be a vector or a matrix. MODELFUNC is a 
%   function describing the data (Y). Starting with the parameter values passed
%   in XPAR, the parameters of the MODELFUNC are optimized using the criterium above.   
%   The algorithm uses the Levenberg-Marquardt method.        
%
%   Parameters:
%   X           - X-data (matrix or vector)
%   Y           - Column vector with Y-data
%   MODELFUNC   - Model function describing the data
%   XPAR        - Column vecor with startvalues for the parameters
%                 of MODELFUNC
%   DERIVFUNC   - (optional) Function describing derivative of MODELFUNC with
%                 respect to its parameters. The partial derivatives
%                 of the MODELFUNC with respect to its parameters.
%                 should be stored in columns.
%   OPTIONS     - (optional)
%                   OPTIONS(1)  =  1 print the intermediate results.
%                               =  2 print & plot intermediate results.
%                               =  3 print only an iteration number
%                               =  0 No output
%                               = -1 Suppress warnings
%                    (Default value: 0)
%                   OPTIONS(2)  =  rel. change of residual sum of squares
%                                  used to halt optimalization.
%                                  A zero indicates that this
%                                  stop criterium wil NOT be used.
%                    (Default value: 1e-4)            
%                   OPTIONS(3)  =  rel. change of residual sum of squares
%                                  used to halt optimalization.
%                                  A zero indicates that this
%                                  stop criterium wil NOT be used.
%                    (Default value: 1e-2)
%                   OPTIONS(4)  =  1 Compare analytically and numerically 
%                                    determined derivatives.
%                                  0 Do not compare them.
%                    (Default value: 0)
%                   OPTIONS(5)  =  Maximum number of evaluations of MODELFUNC.
%                    (Default value: 10000)
%                   OPTIONS(6)  =  Percent of change in the parameters to start with. 
%                    (Default value: 1)
%                   OPTIONS(7)  =  Start value of lambda.
%                                   (This value is used in Levenberg-Marquardt method)
%                    (Default value: 1e-3)
%                   OPTIONS(8)  =  Maximum value of lambda.
%                                   (this value is used in Levenberg-Marquardt method)
%                    (Default value: 1e12)
%                   OPTIONS(9)  =  Minimum value of lambda.
%                                   (used in Levenberg-Marquardt method)
%                    (Default value: 1e-30)
%   P1-P9       - (optional)
%                 Parameters are passed directly to MODELFUNC en DERIVFUNC
%                 Empty arguments ([]) are ignored.
%
%   Returned parameters:
%   OptPar         - A column vector with the optimal parameters.
%   sigma       - A scalar with the residual standard deviation.
%   cormat      - The estimated correlation matrix of the parameters.
%   status      -    1 : Optimal parameter set reached.
%                   -1 : Maximum lambda reached.
%                   -2 : Maximum number of MODELFUNC evaluations.
%                        reached.
%                   -3 : Minimum lambda reached. 
%                   -4 : Inifinite parameter values detected.
%   response    - The response of MODELFUNC using the optimal parameters.
%   DetHessian  - Vector with subsequent values of the determinant of Hessian matrix.
%                 each time the algorithm thinks the fit is converging the determinant
%                 of the Hessian matrix is stored.

% ------------Argument checking and initialization----------------

if ( nargin < 4 )
    error('Call is: [OptPar,sigma,cormat,status,response,DetHessian] = nlr(X,Y,MODELFUNC,XPAR,OPTIONS,DERIVFUNC,P1,P2,P3,P4,P5,P6,P7,P8,P9)')
end

% Make Y and XPAR column vectors.
xdata   = X;   
ydata   = Y(:);
OptPar  = XPAR(:);
npars   = length(OptPar);   % Number of parameters

% modelstr is a string containing call to model function
modelstr = [MODELFUNC];
if (~any(MODELFUNC < 48))
    modelstr=[modelstr, '(xdata,OptPar'];
    % Read the addtional parameters P1,P2,...,P9
    for (i=1:nargin - 6)
        modelstr = [modelstr,',P',int2str(i)];
    end
    modelstr = [modelstr, ')'];
end

if ( nargin < 5 )
    OPTIONS = [];
end
OPTIONS = NLRopts(OPTIONS);

if ( nargin < 6 )
    DERIVFUNC = [];
end

if ( isempty(DERIVFUNC) )
    analderiv = 0;
else
    if ( ~isstr(DERIVFUNC) )
        error('nlr: DERIVFUNC should a string')
    end
    if (length(DERIVFUNC) > 0)
        analderiv = 1;
    else
        analderiv = 0;
    end
end;

if (analderiv)
    derivstr = [DERIVFUNC];
    if (~any(DERIVFUNC<48))
        derivstr = [derivstr, '(xdata,OptPar'];
        % Read additional parameters P1,P2,..,P9
        for (i=1:nargin - 6)
            derivstr = [derivstr,',P',int2str(i)];
        end
        derivstr = [derivstr, ')'];
    end
end
actpar    = OptPar;
compare   = (OPTIONS(4) == 1); 
if ( ~analderiv | compare )
    difpar          = 1e-6 * ones(length(actpar),1);
    index           = (actpar ~= 0);
    difpar(index)   = OPTIONS(6) * actpar(index) / 100.;
end

response        = eval(modelstr);     % Response of model function.
response        = response(:);
nfun            = length(response);
gradient        = zeros(npars,nfun);  % gradient is a matrix with derivatives of MODELFUNC
                                      % to its parameters.
modelcnt        = 1;                  % Number of MODELFUNC evaluations.

% Evaluate options
DisplayIntRes   = (OPTIONS(1) == 1);    
DisplayPlot     = (OPTIONS(1) == 2);
DisplayCycle    = (OPTIONS(1) == 3);
DisplayMes      = (OPTIONS(1) == -1);
Display         = ( DisplayIntRes | DisplayPlot | DisplayCycle );
if ( Display )
    PRparFormat   = ['%11.',num2str(abs(log10(abs(OPTIONS(2)))) + 1),'e '];
    PRssFormat    = ['%11.',num2str(abs(log10(abs(OPTIONS(3)))) + 1),'e '];
end
Display         = ( Display | DisplayCycle );
PARprecision    = OPTIONS(2);  % Tolerance parameters.
SSprecision     = OPTIONS(3);  % Tolerance residual sum of squares.
lambda          = OPTIONS(7);  % Start value for lambda.
maxlambda       = OPTIONS(8);  % Maximal lambda. 
minlambda       = OPTIONS(9);  % Minimal lambda.
maxmodelcnt     = OPTIONS(5);  % Maximal number of MODELFUNC evaluations.

Residuals       = (ydata - response);
ssact           = Residuals' * Residuals;                   % Actual sum of squares of residuals.
ssnew           = 1.1 * ssact;                               
sigma           = sqrt(ssact / ( length(ydata) - npars ));  % Standard deviation of residuals.
cormat          = [];
DetHessian      = [];

% ------------Main loop: minimalization of ssact----------------
status          = 0;
cycle           = 0;
infinite        = 0;
if ( DisplayCycle)
    fprintf('cycle: \r\n');
end
while ( status == 0 )
    if ( ~analderiv | compare )
        % Determine the numerical derivative with respect
        % to the parameters of MODELFUNC.
        for ( parcnt=1:npars )
            OptPar(parcnt)      = actpar(parcnt) + difpar(parcnt);
            response1           = eval(modelstr);
            response1           = response1(:);
            OptPar(parcnt)      = actpar(parcnt) - difpar(parcnt);
            response2           = eval(modelstr);
            response2           = response2(:);
            gradient(parcnt,:)  = (response1 - response2)'/(2 * difpar(parcnt));
        end
        modelcnt = modelcnt + npars;

        if (compare == 1)
            % Check if numerically and analytically
            % determined gradient are the same.
            gradientNUM         = gradient;
            OptPar              = actpar;
            gradient            = eval(derivstr);
            NLRcomp(gradientNUM,gradient, derivstr);
            compare             = 0;
        end
    else
        % Analytically determination of derivatives with respect
        % to the parameters of MODELFUNC, using: DERIVFUNC
        OptPar      = actpar;
        gradient    = eval(derivstr);
    end
    alfa = gradient * gradient';
    beta = gradient * Residuals;
    
    % Levenberg-Marquardt Method
    while ( ssnew >= ssact & lambda <= maxlambda & modelcnt < maxmodelcnt & ~infinite)
        alfan    = alfa + lambda * eye(npars,npars);
        difpar   = inv(alfan) * beta;
        OptPar   = actpar + difpar;
        infinite = ( any(isnan(OptPar) | isinf(OptPar)) );
        if ( ~infinite )
            response    = eval(modelstr);
            modelcnt    = modelcnt + 1;
            Residuals   = ydata - response;
            ssnew       = Residuals' * Residuals;
            if ( ssnew >= ssact )
                lambda = lambda * 10;
            end;    
        end
    end;
    if ( any(Residuals ~= 0) ) 
        if (lambda > maxlambda ) 
            status = -1;
            if ( DisplayMes ) 
                fprintf('lambda must be <= %8.1e\n',maxlambda);
            end
        end
        if ( modelcnt > maxmodelcnt )
            status = -2;
            if ( DisplayMes ) 
                fprintf('Too many function evaluations (> %8d)\n',maxmodelcntax);
            end

        end
        if ( infinite )
            status = -4;
            if ( DisplayMes ) 
                fprintf('Inifinite parameter values detected\n');
            end
        end
    end
    if ( ssnew < ssact & status ~= -4)
        % Check if the tolerance criteria are satisfied
        status0 = (ssnew == 0 );
        status1 = (( abs( ssnew  / ssact - 1) < SSprecision  )| SSprecision  == 0 );
        status2 = ( all( abs( actpar./ OptPar - ones(npars,1) ) < PARprecision ) | PARprecision == 0 );
        status  = status0 | (status1 & status2);
        actpar  = OptPar;
        ssact   = ssnew;
        sigma   = sqrt(ssact / ( length(ydata) - npars ));
        cormat  = inv(alfan);
    end
    if ( status == 0 )
        % The algorithm is approaching the optimum, continu with optimisation
        lambda      = lambda / 10;
        DetHessian  = [DetHessian,det(alfan)];
        cycle       = cycle + 1;
        if ( lambda < minlambda )
            status = -3;
            if ( DisplayMes ) 
                fprintf('lambda must be >= %8.1e\n',minlambda);
            end
        end

        if ( Display ) 
            % Print intermediate results
            if ( DisplayPlot | DisplayIntRes ) 
                fprintf('%4d |',cycle);
                for ( iprint= 1:length(actpar) )
                    fprintf(PRparFormat,actpar(iprint));
                end
                fprintf(' | ');
                fprintf(PRssFormat',ssnew);
                fprintf(' | ');
                fprintf('%11.3e',lambda);
                fprintf(' |\n');
            end
            if ( DisplayPlot )

                subplot(3,1,[2,3]);
                plot(xdata,ydata,'.')
                hold on 
                OptPar      = actpar;
                response    = eval(modelstr);          
                plot(xdata,response,'r-','linewidth',1)
                Title('Data + fit')
                hold off
                drawnow

                subplot(3,1,1)
                plot(xdata,ydata-response,'.','markersize',8);
                title('Residuals')
                drawnow

                fprintf('Print space to continue\r\n')
                pause
            end

            if ( DisplayCycle )
                fprintf('%4d ',cycle);
                if ( rem(cycle,15) == 0 )
                    fprintf('\r\n');
                end
            end

        end
    end
end 
if ( Display )
    fprintf('\r\n');
end
OptPar = actpar;

